div.seesaw,span.seesawinline {
    display: none;
}
div.seesaw.show {
    display: block;
}
span.seesawinline.show {
    display: inline;
}

/*
 * The following counteract the effect of MoinMoin removing display
 * (by setting it to "") when unhiding comments.
 */
div.comment {
    display: block;
}
span.comment {
    display: inline;
}
