(function () {
    var head = document.getElementsByTagName("head")[0];
    if (head) {
	var style = document.createElement("link");
	style.rel = "stylesheet";
	style.type = "text/css";
	style.href = "/jscript/seesawnojump.css";
	head.appendChild(style);
   }
}());

